@@ -1,9 +1,12 @@
 import unittest
+import pytest
 
 from PyFoam.Basics.FoamFileGenerator import FoamFileGenerator,makeString,FoamFileGeneratorError
 from PyFoam.Basics.DataStructures import DictProxy,TupleProxy,Unparsed,UnparsedList,BoolProxy
 from PyFoam.RunDictionary.ParsedParameterFile import ParsedParameterFile,FoamStringParser
 
+from PyFoam.Error import FatalErrorPyFoamException
+
 from PyFoam.FoamInformation import oldTutorialStructure,foamTutorials,foamVersionNumber,foamFork
 
 from os import system,path,remove
@@ -20,8 +23,6 @@ from PyFoam.Error import error
 if PY3:
     long=int
 
-theSuite=unittest.TestSuite()
-
 def damBreakTutorial():
     prefix=foamTutorials()
     if oldTutorialStructure():
@@ -132,6 +133,7 @@ class FoamFileGeneratorTest(unittest.TestCase):
         d["a"]=False
         g=FoamFileGenerator(d)
         self.assertEqual(str(g),"b yes;\na no;\n")
+        self.assertEqual(g.makeString(firstLevel=True),"b yes;\na no;\n")
 
     def testMakeDictionaryRedirect(self):
         val="""
@@ -193,8 +195,10 @@ c {
         self.assertEqual(str(g),"a yes;\n")
         g=FoamFileGenerator({'a':BoolProxy(True,textual="on")})
         self.assertEqual(str(g),"a yes;\n")
-
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorTest,"test"))
+        g=FoamFileGenerator(BoolProxy(True))
+        self.assertEqual(str(g),"yes")
+        g=FoamFileGenerator(BoolProxy(False))
+        self.assertEqual(str(g),"no")
 
 class FoamFileGeneratorUnparsed(unittest.TestCase):
     def testUnparsed(self):
@@ -212,8 +216,6 @@ class FoamFileGeneratorUnparsed(unittest.TestCase):
         g=FoamFileGenerator({"a":Unparsed(text),"b":"nix"})
         self.assertEqual(str(g),"a "+text+";\nb nix;\n")
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorUnparsed,"test"))
-
 class FoamFileGeneratorUnparsedList(unittest.TestCase):
     def testUnparsedList(self):
         content="1\n2\n3\n4\n5\n6"
@@ -228,8 +230,6 @@ class FoamFileGeneratorUnparsedList(unittest.TestCase):
         g=FoamFileGenerator({"a":UnparsedList(6,content),"b":"nix"})
         self.assertEqual(str(g),"a\n  6 ("+content+"\n  );\nb nix;\n")
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorUnparsedList,"test"))
-
 class FoamFileGeneratorRoundtrip(unittest.TestCase):
     def setUp(self):
         self.theFile=mktemp()
@@ -239,6 +239,7 @@ class FoamFileGeneratorRoundtrip(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile)
         data1=deepcopy(test.content)
@@ -247,8 +248,6 @@ class FoamFileGeneratorRoundtrip(unittest.TestCase):
         test2=ParsedParameterFile(self.theFile)
         self.assertEqual(data1,test2.content)
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorRoundtrip,"test"))
-
 class FoamFileGeneratorRoundtrip2(unittest.TestCase):
     def setUp(self):
         self.theFile=mktemp()
@@ -257,6 +256,7 @@ class FoamFileGeneratorRoundtrip2(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile)
         data1=deepcopy(test.content)
@@ -265,8 +265,6 @@ class FoamFileGeneratorRoundtrip2(unittest.TestCase):
         test2=ParsedParameterFile(self.theFile)
         self.assertEqual(data1,test2.content)
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorRoundtrip2,"test"))
-
 class FoamFileGeneratorRoundtripZipped(unittest.TestCase):
     def setUp(self):
         self.theFile=mktemp()
@@ -276,6 +274,7 @@ class FoamFileGeneratorRoundtripZipped(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile+".gz")
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile)
         data1=deepcopy(test.content)
@@ -286,8 +285,6 @@ class FoamFileGeneratorRoundtripZipped(unittest.TestCase):
         test3=ParsedParameterFile(self.theFile+".gz")
         self.assertEqual(data1,test3.content)
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorRoundtripZipped,"test"))
-
 class FoamFileGeneratorRoundtrip2(unittest.TestCase):
     def setUp(self):
         self.theFile=mktemp()
@@ -299,6 +296,7 @@ class FoamFileGeneratorRoundtrip2(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile)
         data1=deepcopy(test.content)
@@ -307,8 +305,6 @@ class FoamFileGeneratorRoundtrip2(unittest.TestCase):
         test2=ParsedParameterFile(self.theFile)
         self.assertEqual(data1,test2.content)
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorRoundtrip2,"test"))
-
 class FoamFileGeneratorRoundtrip3(unittest.TestCase):
     def setUp(self):
         self.theFile=mktemp()
@@ -321,6 +317,7 @@ class FoamFileGeneratorRoundtrip3(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile)
         data1=deepcopy(test.content)
@@ -329,8 +326,6 @@ class FoamFileGeneratorRoundtrip3(unittest.TestCase):
         test2=ParsedParameterFile(self.theFile)
         self.assertEqual(data1,test2.content)
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorRoundtrip3,"test"))
-
 class FoamFileGeneratorRoundtrip4(unittest.TestCase):
     def setUp(self):
         self.theFile=mktemp()
@@ -339,6 +334,7 @@ class FoamFileGeneratorRoundtrip4(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile)
         data1=deepcopy(test.content)
@@ -347,8 +343,6 @@ class FoamFileGeneratorRoundtrip4(unittest.TestCase):
         test2=ParsedParameterFile(self.theFile)
         self.assertEqual(data1,test2.content)
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorRoundtrip4,"test"))
-
 class FoamFileGeneratorRoundtrip5(unittest.TestCase):
     def setUp(self):
         self.theFile=mktemp()
@@ -361,6 +355,7 @@ class FoamFileGeneratorRoundtrip5(unittest.TestCase):
             # there is no appropriate volSymmTensorField-file in 2.x
             remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         if foamVersionNumber()>=(2,):
             # there is no appropriate volSymmTensorField-file in 2.x
@@ -373,8 +368,6 @@ class FoamFileGeneratorRoundtrip5(unittest.TestCase):
         test2=ParsedParameterFile(self.theFile)
         self.assertEqual(data1,test2.content)
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorRoundtrip5,"test"))
-
 class FoamFileGeneratorRoundtripLongList(unittest.TestCase):
     def setUp(self):
         self.theFile=mktemp()
@@ -383,11 +376,15 @@ class FoamFileGeneratorRoundtripLongList(unittest.TestCase):
             alphaName="alpha.air"
         elif foamVersionNumber()>=(2,):
             alphaName="alpha1"
-        copyfile(path.join(bubbleColumnTutorial(),"0",alphaName),self.theFile)
+        try:
+            copyfile(path.join(bubbleColumnTutorial(),"0",alphaName),self.theFile)
+        except IOError:
+            copyfile(path.join(bubbleColumnTutorial(),"0.orig",alphaName),self.theFile)
 
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile,listLengthUnparsed=100)
         data1=deepcopy(test.content)
@@ -396,8 +393,6 @@ class FoamFileGeneratorRoundtripLongList(unittest.TestCase):
         test2=ParsedParameterFile(self.theFile,listLengthUnparsed=100)
         self.assertEqual(data1,test2.content)
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorRoundtripLongList,"test"))
-
 class FoamFileGeneratorRoundtripLongList2(unittest.TestCase):
     def setUp(self):
         self.theFile=mktemp()
@@ -407,11 +402,15 @@ class FoamFileGeneratorRoundtripLongList2(unittest.TestCase):
         elif foamVersionNumber()>=(2,):
             UName="U1"
 
-        copyfile(path.join(bubbleColumnTutorial(),"0",UName),self.theFile)
+        try:
+            copyfile(path.join(bubbleColumnTutorial(),"0",UName),self.theFile)
+        except IOError:
+            copyfile(path.join(bubbleColumnTutorial(),"0.orig",UName),self.theFile)
 
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile,listLengthUnparsed=100)
         data1=deepcopy(test.content)
@@ -420,14 +419,22 @@ class FoamFileGeneratorRoundtripLongList2(unittest.TestCase):
         test2=ParsedParameterFile(self.theFile,listLengthUnparsed=100)
         self.assertEqual(data1,test2.content)
 
-theSuite.addTest(unittest.makeSuite(FoamFileGeneratorRoundtripLongList2,"test"))
-
 class MakeStringFunction(unittest.TestCase):
     def testSingleTuple(self):
         self.assertEqual(makeString( (2,3) ),"  2   3 ")
+
     def testSingleTupleProxy(self):
         self.assertEqual(makeString( TupleProxy((2,3)) ),"  2   3 ")
 
+    def testTupleWithList(self):
+        self.assertEqual(makeString( (2,[3,4]) ),"  2    (\n    3\n    4\n  )\n")
+
+    def testTupleWithDict(self):
+        self.assertEqual(makeString( (2,{"a":2}) ),"  2 {\n    a 2;\n  } ")
+
+    def testTupleInDict(self):
+        self.assertEqual(makeString({"a":(2,3)} ),"a     2     3 ;\n")
+
     def testSingleList(self):
         self.assertEqual(makeString( [2,3] ),"(\n  2\n  3\n)\n")
 
@@ -446,7 +453,10 @@ class MakeStringFunction(unittest.TestCase):
         p1=FoamStringParser('test  nonuniform 0();')
         self.assertEqual(str(p1),"test nonuniform 0\n(\n)\n;\n")
 
-theSuite.addTest(unittest.makeSuite(MakeStringFunction,"test"))
+class QuotedString(unittest.TestCase):
+    def testSingleQuote(self):
+        self.assertEqual(makeString( "heres ; nothing" ),'"heres ; nothing"')
+        self.assertEqual(makeString( 'heres ; "nothing"' ),"'heres ; \"nothing\"'")
 
 class IncludeFilesRoundTrip(unittest.TestCase):
     def setUp(self):
@@ -465,6 +475,7 @@ class IncludeFilesRoundTrip(unittest.TestCase):
     def tearDown(self):
         rmtree(self.theDir)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile,listLengthUnparsed=100)
         data1=deepcopy(test.content)
@@ -473,6 +484,7 @@ class IncludeFilesRoundTrip(unittest.TestCase):
         test2=ParsedParameterFile(self.theFile,listLengthUnparsed=100)
         self.assertEqual(data1,test2.content)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorialWithMacros(self):
         test=ParsedParameterFile(self.theFile,
                                  listLengthUnparsed=100,
@@ -491,9 +503,6 @@ class IncludeFilesRoundTrip(unittest.TestCase):
                 else:
                     self.compareData(d1[k],d2[k])
 
-if foamVersionNumber()>=(1,6):
-    theSuite.addTest(unittest.makeSuite(IncludeFilesRoundTrip,"test"))
-
 class CodeStreamRoundTrip(unittest.TestCase):
     def setUp(self):
         self.theDir=mkdtemp()
@@ -504,6 +513,7 @@ class CodeStreamRoundTrip(unittest.TestCase):
     def tearDown(self):
         rmtree(self.theDir)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testBasicInclude(self):
         if foamVersionNumber()<(2,):
             return
@@ -514,5 +524,8 @@ class CodeStreamRoundTrip(unittest.TestCase):
         test2=ParsedParameterFile(self.theFile)
         self.assertEqual(data1,test2.content)
 
-if foamVersionNumber()>=(2,):
-    theSuite.addTest(unittest.makeSuite(CodeStreamRoundTrip,"test"))
+class ErrorRaised(unittest.TestCase):
+    def testUnknownType(self):
+        with pytest.raises(FatalErrorPyFoamException):
+            g=FoamFileGenerator(set())
+            self.assertEqual(str(g),"yes")
