@@ -1,5 +1,6 @@
+import pytest
 import unittest
 
-from PyFoam.Infrastructure.Blink1 import Blink1
+requests = pytest.importorskip("requests")
 
-theSuite=unittest.TestSuite()
+from PyFoam.Infrastructure.Blink1 import Blink1
