@@ -1,11 +1,16 @@
 import unittest
+import pytest
 
 from PyFoam.FoamInformation import oldTutorialStructure,foamTutorials,foamVersionNumber,foamFork
 from os import path,remove
 from tempfile import mktemp,mkdtemp
 from shutil import copyfile,rmtree,copytree
 
-from PyFoam.RunDictionary.ParsedParameterFile import FoamStringParser,ParsedParameterFile,ParsedBoundaryDict,DictProxy,TupleProxy,PyFoamParserError,WriteParameterFile
+from PyFoam.RunDictionary.ParsedParameterFile import FoamStringParser, \
+    ParsedParameterFile, ParsedBoundaryDict, DictProxy, TupleProxy, \
+    PyFoamParserError, WriteParameterFile, FoamFileParser
+
+from PyFoam.Error import FatalErrorPyFoamException
 
 from PyFoam.Basics.FoamFileGenerator import Vector,Dimension,Field,Tensor,SymmTensor,Codestream
 
@@ -78,7 +83,15 @@ def potentialCylinderTutorial():
         prefix=path.join(prefix,"basic","potentialFoam")
     return path.join(prefix,"cylinder")
 
-theSuite=unittest.TestSuite()
+def planarContractionTutorial():
+    prefix=foamTutorials()
+
+    return path.join(prefix,"incompressible","pimpleFoam","laminar","planarContraction")
+
+def backwardFacingStep2DTutorial():
+    prefix=foamTutorials()
+
+    return path.join(prefix,"incompressible","simpleFoam","backwardFacingStep2D")
 
 class FoamStringParserTest(unittest.TestCase):
 
@@ -148,34 +161,34 @@ der name
     def testParseFieldUniform(self):
         p1=FoamStringParser('test  uniform 42;')
         self.assertEqual(type(p1["test"]),Field)
-        self.assert_(p1["test"].isUniform())
+        self.assertTrue(p1["test"].isUniform())
 
     def testParseFieldNonniform(self):
         p1=FoamStringParser('test  nonuniform 4(42 66 34 44);')
         self.assertEqual(type(p1["test"]),Field)
-        self.assert_(not p1["test"].isUniform())
+        self.assertTrue(not p1["test"].isUniform())
 
     def testParseFieldNonniformLengthThree(self):
         p1=FoamStringParser('test  nonuniform 3(42 66 34);')
         self.assertEqual(type(p1["test"]),Field)
-        self.assert_(not p1["test"].isUniform())
+        self.assertTrue(not p1["test"].isUniform())
 
     def testParseFieldNonniformLengthZero(self):
         p1=FoamStringParser('test  nonuniform 0();')
         self.assertEqual(type(p1["test"]),Field)
-        self.assert_(not p1["test"].isUniform())
+        self.assertTrue(not p1["test"].isUniform())
 
     def testListPrefixUniform(self):
         p=FoamStringParser("test 10{42.5};")
         self.assertEqual(len(p["test"]),10)
         self.assertEqual(type(p["test"]),Field)
-        self.assert_(p["test"].isUniform())
+        self.assertTrue(p["test"].isUniform())
 
     def testListPrefixUniformVector(self):
         p=FoamStringParser("test 10{(1 2 3)};")
         self.assertEqual(len(p["test"]),10)
         self.assertEqual(type(p["test"]),Field)
-        self.assert_(p["test"].isUniform())
+        self.assertTrue(p["test"].isUniform())
 
     def testListPrefixNested(self):
         p=FoamStringParser("test 3 ( 3{0.} 3 (1 1 1) 3 (2 2 2));")
@@ -295,6 +308,17 @@ der name
         p1=FoamStringParser("test [m s^-1];")
         self.assertEqual(p1["test"].__class__,Dimension)
 
+    def testDataDimensionedScalarOld(self):
+        p1=FoamStringParser("nu    nu     [0 2 -1 0 0 0 0] 1e-05;")
+        self.assertEqual(p1["nu"][0],"nu")
+        self.assertEqual(p1["nu"][1].__class__,Dimension)
+        self.assertAlmostEqual(p1["nu"][2],1.e-5,6)
+
+    def testDataDimensionedScalarNew(self):
+        p1=FoamStringParser("nu       [0 2 -1 0 0 0 0] 1e-05;")
+        self.assertEqual(p1["nu"][0].__class__,Dimension)
+        self.assertAlmostEqual(p1["nu"][1],1.e-5,6)
+
     def testStringConversion(self):
         p1=FoamStringParser("test dings 2;")
         self.assertEqual(str(p1),"test dings     2 ;\n")
@@ -404,7 +428,7 @@ nix "a=3+x;b=4;";
         p1=FoamStringParser("nix 2; test  3;")
         self.assertEqual(p1["nix"],2)
         del p1["nix"]
-        self.assert_("nix" not in p1)
+        self.assertTrue("nix" not in p1)
 
     def testSubstitute1(self):
         p1=FoamStringParser("nix $da;")
@@ -441,7 +465,30 @@ nix "a=3+x;b=4;";
     def testInputMode4(self):
         p1=FoamStringParser('#inputMode default\n')
 
-theSuite.addTest(unittest.makeSuite(FoamStringParserTest,"test"))
+
+class CheckDuplicateHandling(unittest.TestCase):
+    def setUp(self):
+        self.txt="""
+a 1;
+b 2;
+a 3;
+"""
+
+    def testNoCheckDuplicate(self):
+        p1=FoamStringParser(self.txt)
+        assert p1["a"]==3
+
+    def testCheckDuplicate(self):
+        p1=FoamStringParser(self.txt,
+                            duplicateCheck=True)
+        assert p1["a"]==3
+
+    def testCheckDuplicateFail(self):
+        with pytest.raises(FatalErrorPyFoamException):
+            p1=FoamStringParser(self.txt,
+                                duplicateCheck=True,
+                                duplicateFail=True)
+            assert p1["a"]==3
 
 class ParsedParameterDictionaryMacroExpansion(unittest.TestCase):
     def testSimpleSubst(self):
@@ -524,7 +571,66 @@ c ${${b}}; // returns 10, since $b returns 'a', and $a returns 10
         self.assertEqual(p1["b"],"a")
         self.assertEqual(p1["c"],"${${b}}")
 
-theSuite.addTest(unittest.makeSuite(ParsedParameterDictionaryMacroExpansion,"test"))
+
+class FoamFileParser_FileHeaderTest(unittest.TestCase):
+    def testRegularOpenFOAMHeader(self):
+        p = FoamFileParser("""/*--------------------------------*- C++ -*----------------------------------*\
+| =========                 |                                                 |
+| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
+|  \\    /   O peration     | Version:  v2012                                 |
+|   \\  /    A nd           | Website:  www.openfoam.com                      |
+|    \\/     M anipulation  |                                                 |
+\*---------------------------------------------------------------------------*/
+FoamFile
+{
+    version     2.0;
+    format      ascii;
+    class       dictionary;
+    location    "system";
+    object      controlDict;
+}
+// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
+
+application     laplacianFoam;
+
+""")
+        assert len(p.getData()) == 1
+        assert p.getData() == { "application": "laplacianFoam" }
+
+        assert len(p.getHeader()) == 5
+        assert p.getHeader()["object"] == "controlDict"
+
+    def testExtendedOpenFOAMHeader(self):
+        p = FoamFileParser("""/*--------------------------------*- C++ -*----------------------------------*\
+| =========                 |                                                 |
+| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
+|  \\    /   O peration     | Version:  v2012                                 |
+|   \\  /    A nd           | Website:  www.openfoam.com                      |
+|    \\/     M anipulation  |                                                 |
+\*---------------------------------------------------------------------------*/
+FoamFile
+{
+    version     2.0;
+    format      ascii;
+    class       dictionary;
+    location    "system";
+    object      controlDict;
+    meta {
+        nProcs 4;
+        collated yes;
+    }
+}
+// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
+
+application     laplacianFoam;
+
+""")
+        assert len(p.getData()) == 1
+        assert p.getData() == { "application": "laplacianFoam" }
+
+        assert len(p.getHeader()) == 6
+        assert p.getHeader()["object"] == "controlDict"
+        assert p.getHeader()["meta"] == { "nProcs": 4, "collated": True}
 
 class ParsedBoundaryDictTest(unittest.TestCase):
     def setUp(self):
@@ -534,12 +640,12 @@ class ParsedBoundaryDictTest(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedBoundaryDict(self.theFile)
         self.assertEqual(len(test.content),4)
-        self.assert_("inlet" in test)
+        self.assertTrue("inlet" in test)
 
-theSuite.addTest(unittest.makeSuite(ParsedBoundaryDict,"test"))
 
 class ParsedParameterFileTest(unittest.TestCase):
     def setUp(self):
@@ -557,6 +663,7 @@ class ParsedParameterFileTest(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile)
         if foamVersionNumber()<(1,5):
@@ -578,7 +685,6 @@ class ParsedParameterFileTest(unittest.TestCase):
         self.assertEqual(len(test),nrTurbModels)
         self.assertEqual(test[model],"kEpsilon")
 
-theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest,"test"))
 
 class ParsedParameterFileTest2(unittest.TestCase):
     def setUp(self):
@@ -589,16 +695,19 @@ class ParsedParameterFileTest2(unittest.TestCase):
             extension=".orig"
         else:
             extension=".org"
-        copyfile(path.join(damBreakTutorial(),"0",gammaName()+extension),self.theFile)
+        try:
+            copyfile(path.join(damBreakTutorial(),"0",gammaName()+extension),self.theFile)
+        except IOError:
+            copyfile(path.join(damBreakTutorial(),"0"+extension,gammaName()),self.theFile)
 
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile)
         self.assertEqual(len(test["boundaryField"]),5)
 
-theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest2,"test"))
 
 class ParsedParameterFileTest3(unittest.TestCase):
     def setUp(self):
@@ -611,6 +720,7 @@ class ParsedParameterFileTest3(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile)
         if foamVersionNumber()>=(3,):
@@ -620,7 +730,6 @@ class ParsedParameterFileTest3(unittest.TestCase):
         if foamVersionNumber()<(2,):
             self.assertEqual(len(test["boundaryField"]["floor"]["value"].val),400)
 
-theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest3,"test"))
 
 class ParsedParameterFileTest4(unittest.TestCase):
     def setUp(self):
@@ -633,6 +742,7 @@ class ParsedParameterFileTest4(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         test=ParsedParameterFile(self.theFile)
         gradSchemes=1
@@ -646,7 +756,6 @@ class ParsedParameterFileTest4(unittest.TestCase):
         else:
             self.assertEqual(len(test["divSchemes"]["div(phi,ft_b_h_hu)"][2]),divSchemes)
 
-theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest4,"test"))
 
 class ParsedParameterFileTest5(unittest.TestCase):
     def setUp(self):
@@ -660,6 +769,7 @@ class ParsedParameterFileTest5(unittest.TestCase):
             # there is no appropriate volSymmTensorField-file in 2.x
             remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         if foamVersionNumber()>=(2,):
             # there is no appropriate volSymmTensorField-file in 2.x
@@ -673,16 +783,12 @@ class ParsedParameterFileTest5(unittest.TestCase):
         else:
             self.assertEqual(test["internalField"].value().__class__,SymmTensor)
 
-theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest5,"test"))
 
 class ParsedParameterFileTest6(unittest.TestCase):
     def setUp(self):
-        try:
-            if foamVersionNumber()>=(1,6):
-                from nose.plugins.skip import SkipTest
-                raise SkipTest()
-        except ImportError:
-            pass
+        if foamVersionNumber()>=(1,6):
+            import pytest
+            pytest.skip("aachenBomb-tutorial no longer existing")
 
         self.theFile=mktemp()
         copyfile(path.join(dieselAachenTutorial(),"0","spray"),self.theFile)
@@ -690,15 +796,13 @@ class ParsedParameterFileTest6(unittest.TestCase):
     def tearDown(self):
         remove(self.theFile)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadTutorial(self):
         try:
             test=ParsedParameterFile(self.theFile)
         except PyFoamParserError:
             pass
 
-if foamVersionNumber()<(1,5):
-    theSuite.addTest(unittest.makeSuite(ParsedParameterFileTest6,"test"))
-
 class ParsedParameterFileIncludeTest(unittest.TestCase):
     def setUp(self):
         if oldTutorialStructure():
@@ -712,14 +816,38 @@ class ParsedParameterFileIncludeTest(unittest.TestCase):
     def tearDown(self):
         pass
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testBasicInclude(self):
         test=ParsedParameterFile(self.fileName,doMacroExpansion=True)
         self.assertEqual(test["pressure"],0)
         self.assertEqual("upperWall" in test["boundaryField"],True)
         self.assertEqual(test["boundaryField"]["upperWall"]["type"],"slip")
 
-if foamVersionNumber()>=(1,6):
-    theSuite.addTest(unittest.makeSuite(ParsedParameterFileIncludeTest,"test"))
+class ParsedParameterFileIncludeFuncTest(unittest.TestCase):
+    def setUp(self):
+        self.fileName=path.join(planarContractionTutorial(),"system","controlDict")
+
+    def tearDown(self):
+        pass
+
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
+    def testBasicIncludeFunc(self):
+        test=ParsedParameterFile(self.fileName,doMacroExpansion=True)
+        self.assertEqual(test["functions"]["graphs"]["fields"],["U"])
+        self.assertEqual(test["functions"]["graphs"]["writeControl"],"writeTime")
+
+class ParsedParameterFileIncludeFuncQuotedTest(unittest.TestCase):
+    def setUp(self):
+        self.fileName=path.join(backwardFacingStep2DTutorial(),"system","controlDict")
+
+    def tearDown(self):
+        pass
+
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
+    def testBasicIncludeFunc(self):
+        test=ParsedParameterFile(self.fileName,doMacroExpansion=True)
+        self.assertEqual(test["functions"]["sampleCp"]["fields"],["cp"])
+        self.assertEqual(test["functions"]["wallShearStress"]["writeControl"],"writeTime")
 
 class ParsedParameterFileCodeStreamTest(unittest.TestCase):
     def setUp(self):
@@ -728,6 +856,7 @@ class ParsedParameterFileCodeStreamTest(unittest.TestCase):
     def tearDown(self):
         pass
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testBasicInclude(self):
         test=ParsedParameterFile(self.fileName)
         if foamVersionNumber()<(2,):
@@ -740,9 +869,6 @@ class ParsedParameterFileCodeStreamTest(unittest.TestCase):
             code="code"
         self.assertEqual(type(test["functions"][key][code]),Codestream)
 
-if foamVersionNumber()>=(2,):
-    theSuite.addTest(unittest.makeSuite(ParsedParameterFileCodeStreamTest,"test"))
-
 class ParsedParameterFileRedirectDictionary(unittest.TestCase):
     def setUp(self):
         self.data="""
@@ -817,7 +943,6 @@ dictC {
         self.assertEqual(data["dictA"]["f"],17)
         self.assertEqual(data["dictA"]["f"],17)
 
-theSuite.addTest(unittest.makeSuite(ParsedParameterFileRedirectDictionary,"test"))
 
 class ParsedParameterFileDictionaryNested(unittest.TestCase):
     def setUp(self):
@@ -861,7 +986,6 @@ dictA {
             data["dictA"]["dictB"]["dictC"]["dictD"]["e"],
             data["dictA"]["dictB"]["dictC"]["d"])
 
-theSuite.addTest(unittest.makeSuite(ParsedParameterFileDictionaryNested,"test"))
 
 class ParsedParameterFileDictionaryNestedCopy(unittest.TestCase):
     def setUp(self):
@@ -891,7 +1015,6 @@ dictB {
         self.assertEqual(data["dictA"]["a"],1)
         self.assertEqual(42,data["dictB"]["d"]["a"])
 
-theSuite.addTest(unittest.makeSuite(ParsedParameterFileDictionaryNestedCopy,"test"))
 
 class WriteParameterFileTest(unittest.TestCase):
     def setUp(self):
@@ -928,7 +1051,6 @@ c 4;
 
 """)
 
-theSuite.addTest(unittest.makeSuite(WriteParameterFileTest,"test"))
 
 class ReadIncludeAndMacroExpansionTest(unittest.TestCase):
     def setUp(self):
@@ -945,12 +1067,14 @@ class ReadIncludeAndMacroExpansionTest(unittest.TestCase):
     def tearDown(self):
         rmtree(self.testDir)
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadNoMacroExpansion(self):
         kFile=ParsedParameterFile(path.join(self.dest,"k"))
         self.assertEqual(str(kFile["internalField"]),"uniform $turbulentKE")
         self.assertEqual(kFile["boundaryField"]["lowerWall"]["value"],"$internalField")
         self.assertEqual(kFile["boundaryField"]["motorBikeGroup"]["value"],"$internalField")
 
+    @pytest.mark.skipif(foamTutorials()=='',reason="$FOAM_TUTORIALS is not defined")
     def testReadMacroExpansion(self):
         kFile=ParsedParameterFile(path.join(self.dest,"k"),
                                   doMacroExpansion=True)
@@ -959,8 +1083,6 @@ class ReadIncludeAndMacroExpansionTest(unittest.TestCase):
         self.assertEqual(str(kFile["boundaryField"]["motorBikeGroup"]["value"]),"uniform 0.24")
 
 
-theSuite.addTest(unittest.makeSuite(WriteParameterFileTest,"test"))
-
 class RegexpFindingTests(unittest.TestCase):
     def setUp(self):
         self.parse=FoamStringParser("""
@@ -973,3 +1095,40 @@ abValue 3;
         self.assertEqual(self.parse["aValue"],1)
         self.assertEqual(self.parse["bValue"],2)
         self.assertEqual(self.parse["abValue"],3)
+
+class DictLikeParserBehaviour(unittest.TestCase):
+    def setUp(self):
+        self.data=FoamStringParser("""
+a 1;
+b 2;
+c 3;
+""")
+
+    def testHasItem(self):
+        self.assertTrue("a" in self.data)
+
+    def testGetItem(self):
+        self.assertEqual(self.data["a"],1)
+
+    def testSetItem(self):
+        self.data["a"]=-1
+        self.assertEqual(self.data["a"],-1)
+
+    def testDelItem(self):
+        del self.data["a"]
+        with pytest.raises(KeyError):
+            self.assertEqual(self.data["a"],-1)
+
+    def testIter(self):
+        self.assertEqual([k for k in self.data],["a","b","c"])
+
+class ReadBinaryFiles(unittest.TestCase):
+    def testReadBinaryFile1(self):
+        uFile=ParsedParameterFile(path.join("testdata","U.binary"))
+        assert uFile["boundaryField"]["fixedWalls"]["type"]=="noSlip"
+        assert uFile["internalField"].isBinary()
+
+    def testReadBinaryFile2(self):
+        pFile=ParsedParameterFile(path.join("testdata","p.binary"))
+        assert pFile["boundaryField"]["fixedWalls"]["type"]=="zeroGradient"
+        assert pFile["internalField"].isBinary()
